/* 
 * Proview   Open Source Process Control.
 * Copyright (C) 2005-2016 SSAB EMEA AB.
 *
 * This file is part of Proview.
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation, either version 2 of 
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with Proview. If not, see <http://www.gnu.org/licenses/>
 *
 * Linking Proview statically or dynamically with other modules is
 * making a combined work based on Proview. Thus, the terms and 
 * conditions of the GNU General Public License cover the whole 
 * combination.
 *
 * In addition, as a special exception, the copyright holders of
 * Proview give you permission to, from the build function in the
 * Proview Configurator, combine Proview with modules generated by the
 * Proview PLC Editor to a PLC program, regardless of the license
 * terms of these modules. You may copy and distribute the resulting
 * combined work under the terms of your choice, provided that every 
 * copy of the combined work is accompanied by a complete copy of 
 * the source code of Proview (the version used to produce the 
 * combined work), being distributed under the terms of the GNU 
 * General Public License plus this exception.
 **/

#include "glow_std.h"


#include <stdlib.h>
#include <iostream>
#include <fstream>
#include "glow_text.h"
#include "glow_draw.h"

void GlowText::zoom()
{
  p.zoom();
}

void GlowText::nav_zoom()
{
  p.nav_zoom();
}

void GlowText::print_zoom()
{
  p.print_zoom();
}

void GlowText::traverse( int x, int y)
{
  p.traverse( x, y);
}

void GlowText::save( ofstream& fp, glow_eSaveMode mode)
{
  fp << int(glow_eSave_Text) << endl;
  fp << int(glow_eSave_Text_text_size) << FSPACE << text_size << endl;
  fp << int(glow_eSave_Text_draw_type) << FSPACE << int(draw_type) << endl;
  fp << int(glow_eSave_Text_color_drawtype) << FSPACE << int(color_drawtype) << endl;
  fp << int(glow_eSave_Text_text) << FSPACE << text << endl;
  fp << int(glow_eSave_Text_p) << endl;
  p.save( fp, mode);
  fp << int(glow_eSave_End) << endl;
}

void GlowText::open( ifstream& fp)
{
  int		type;
  int 		end_found = 0;
  char		dummy[40];
  int		tmp;

  for (;;)
  {
    if ( !fp.good()) {
      fp.clear();
      fp.getline( dummy, sizeof(dummy));
      printf( "** Read error GlowText: \"%d %s\"\n", type, dummy);      
    }

    fp >> type;
    switch( type) {
      case glow_eSave_Text: break;
      case glow_eSave_Text_text_size: fp >> text_size; break;
      case glow_eSave_Text_draw_type: fp >> tmp; draw_type = (glow_eDrawType)tmp; break;
      case glow_eSave_Text_color_drawtype: fp >> tmp; color_drawtype = (glow_eDrawType)tmp; break;
      case glow_eSave_Text_text: 
        fp.get();
        fp.getline( text, sizeof(text));
        break;
      case glow_eSave_Text_p: p.open( fp); break;
      case glow_eSave_End: end_found = 1; break;
      default:
        cout << "GlowText:open syntax error" << endl;
        fp.getline( dummy, sizeof(dummy));
    }
    if ( end_found)
      break;
  }
}

void GlowText::draw( GlowWind *w, void *pos, int highlight, int hot, void *node)
{
  int idx = int( w->zoom_factor_y / w->base_zoom_factor * (text_size +4) - 4);
  if ( idx < 0)
    return;
  idx = min( idx, DRAW_TYPE_SIZE-1);
  ctx->gdraw->text( w, p.z_x + ((GlowPoint *)pos)->z_x - w->offset_x, 
		    p.z_y + ((GlowPoint *)pos)->z_y - w->offset_y, text, strlen(text),
		    draw_type, color_drawtype, idx, highlight, 0, glow_eFont_Helvetica,
		    w->zoom_factor_y / w->base_zoom_factor * (8+2*text_size), 0);
}

void GlowText::erase( GlowWind *w, void *pos, int hot, void *node)
{
  int idx = int( w->zoom_factor_y / w->base_zoom_factor * (text_size +4) - 4);
  if ( idx < 0)
    return;
  idx = min( idx, DRAW_TYPE_SIZE-1);
  ctx->gdraw->text_erase( w, p.z_x + ((GlowPoint *)pos)->z_x - w->offset_x, 
			  p.z_y + ((GlowPoint *)pos)->z_y - w->offset_y, text,  strlen(text),
			  draw_type, idx, 0, glow_eFont_Helvetica,
			  w->zoom_factor_y / w->base_zoom_factor * (8+2*text_size), 0);
}

int GlowText::event_handler( GlowWind *w, void *pos, glow_eEvent event, int x, int y,
	void *node)
{
  GlowPoint *p;

  p = (GlowPoint *) pos;
    return 0;
}

void GlowText::get_borders( double pos_x, double pos_y, double *x_right, 
		double *x_left, double *y_high, double *y_low, void *node)
{
}

void GlowText::move( void *pos, double x, double y, int highlight, int hot)
{

  erase( &ctx->mw, pos, hot, NULL);
  erase( &ctx->navw, pos, 0, NULL);
  p.x = x;
  p.y = y;
  zoom();
  nav_zoom();
  draw( &ctx->mw, pos, highlight, hot, NULL);
  draw( &ctx->navw, pos, highlight, 0, NULL);
}

void GlowText::shift( void *pos, double delta_x, double delta_y,
	int highlight, int hot)
{
  erase( &ctx->mw, pos, hot, NULL);
  erase( &ctx->navw, pos, 0, NULL);
  p.x += delta_x;
  p.y += delta_y;
  zoom();
  nav_zoom();

  draw( &ctx->mw, pos, highlight, hot, NULL);
  draw( &ctx->navw, pos, highlight, 0, NULL);
}


